@@ -34,6 +34,7 @@ against the expression given::
   - ``yMin`` (minimum Y coordinate)
   - ``yMax`` (maximum Y coordinate)
   - ``rise`` (difference in Y coordinate between cursive entry and exit)
+  - ``fullwidth`` (``xMax``-``xMin``)
 
 - The second part is a comparison operator (``>=``, ``<=``,
   ``=``, ``<``, or ``>``).
@@ -47,6 +48,11 @@ less than the advance width of the ``space`` glyph::
 
     DefineClass @shorter_than_space = @alpha and (width < width(space));
 
+- As well as testing for glyph metrics, the following other relationships
+  are defined:
+
+  - ``hasanchor(anchorname)`` (true if the glyph has the named anchor)
+  - ``category(base)`` (true if the glyph has the given category)
 
 Binned Definitions
 ------------------
@@ -87,12 +93,15 @@ will take any glyph selector and display its contents on standard error.
 """
 
 import re
-from glyphtools import get_glyph_metrics
+from glyphtools import get_glyph_metrics, bin_glyphs_by_metric
+
 import warnings
 
 
 GRAMMAR = """
-predicate = ws 'and' ws '(' ws <letter+>:metric ws ('>='|'<='|'='|'<'|'>'):comparator ws (<digit+>|bracketed_metric):value ws ')' -> {'predicate': metric, 'comparator': comparator, 'value': value}
+predicate = ws 'and' ws (has_anchor_predicate | category_predicate |'not'?:n ws  '(' ws <letter+>:metric ws ('>='|'<='|'='|'<'|'>'):comparator ws (integer|bracketed_metric):value ws ')' -> {'predicate': metric, 'comparator': comparator, 'value': value, 'inverted': n} )
+has_anchor_predicate = 'not'?:n ws 'hasanchor(' barename:anchor ')' -> {'predicate': 'hasanchor', 'value': anchor["barename"], 'inverted':n }
+category_predicate = 'not'?:n ws 'category(' barename:cat ')' -> {'predicate': 'category', 'value': cat["barenamed"], 'inverted':n }
 bracketed_metric = <letter+>:metric '(' <(letter|digit|"."|"_")+>:glyph ')' -> {'metric': metric, 'glyph': glyph}
 andconjunction = glyphselector:l ws '&' ws primary:r -> {'conjunction': 'and', 'left': l, 'right': r}
 orconjunction = glyphselector:l2 ws '|' ws primary:r2 -> {'conjunction': 'or', 'left': l2, 'right': r2}
@@ -132,36 +141,46 @@ class DefineClass:
 
     @classmethod
     def meets_predicate(self, glyphname, predicate, parser):
+        inverted = predicate["inverted"]
         metric = predicate["predicate"]
-        comp = predicate["comparator"]
-        if isinstance(predicate["value"], dict):
-            v = predicate["value"]
-            testvalue_metrics = get_glyph_metrics(parser.font, v["glyph"])
-            if v["metric"] not in testvalue_metrics:
-                raise ValueError("Unknown metric '%s'" % metric)
-            testvalue = testvalue_metrics[v["metric"]]
+        if metric == "hasanchor":
+            anchor = predicate["value"]
+            truth = glyphname in parser.fontfeatures.anchors and anchor in parser.fontfeatures.anchors[glyphname]
+        elif metric == "category":
+            cat = predicate["value"]
+            truth = parser.font[glyphname].category == cat
         else:
-            testvalue = int(predicate["value"])
-
-        metrics = get_glyph_metrics(parser.font, glyphname)
-        if metric not in metrics:
-            raise ValueError("Unknown metric '%s'" % metric)
-        value = metrics[metric]
-        if comp == ">":
-            return value > testvalue
-        elif comp == "<":
-            return value < testvalue
-        elif comp == ">=":
-            return value >= testvalue
-        elif comp == "<=":
-            return value <= testvalue
-        raise ValueError("Bad comparator (can't happen?)")
+            comp = predicate["comparator"]
+            if isinstance(predicate["value"], dict):
+                v = predicate["value"]
+                testvalue_metrics = get_glyph_metrics(parser.font, v["glyph"])
+                if v["metric"] not in testvalue_metrics:
+                    raise ValueError("Unknown metric '%s'" % metric)
+                testvalue = testvalue_metrics[v["metric"]]
+            else:
+                testvalue = int(predicate["value"])
 
+            metrics = get_glyph_metrics(parser.font, glyphname)
+            if metric not in metrics:
+                raise ValueError("Unknown metric '%s'" % metric)
+            value = metrics[metric]
+            if comp == ">":
+                truth = value > testvalue
+            elif comp == "<":
+                truth = value < testvalue
+            elif comp == ">=":
+                truth = value >= testvalue
+            elif comp == "<=":
+                truth = value <= testvalue
+            else:
+                raise ValueError("Bad comparator %s (can't happen?)" % comp)
+        if inverted:
+            truth = not truth
+        return truth
 
 class DefineClassBinned(DefineClass):
     @classmethod
     def action(self, parser, metric, bincount, classname, definition):
-        from glyphtools import bin_glyphs_by_metric
         glyphs = self.resolve_definition(parser, definition[0])
         predicates = definition[1]
         for p in predicates:
